/*
 * Copyright (C) 2009 Valery Inozemtsev <shrek@altlinux.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details:
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <libudev.h>
#include "alsactl.h"

int force_restore = 1;
int ignore_nocards = 0;
char *statefile = NULL;
char *resfile = NULL;
char resdir[UTIL_PATH_SIZE];

void save_mixer(char *cardnumber)
{
	save_state(statefile, cardnumber);
}

void restore_mixer(char *cardnumber)
{
	if (load_state(statefile, NULL, cardnumber, 1) == 0) {
		creat(resfile, 0644);
	} else {
		unlink(resfile);
		unlink(statefile);
	}
}

void control(char *cardnumber)
{
	char *action = NULL;

	asprintf(&statefile, "%s%s", STATE_FILE, cardnumber);
	asprintf(&resfile, "%s%s%s", resdir, "/controlC", cardnumber);

	action = getenv("ACTION");
	if (action != NULL) {
		if (strcmp(action, "remove") == 0) {
			unlink(resfile);
			return;
		}
	}

	if (access(statefile, F_OK) != 0) {
		set_mixer_default(cardnumber);
		save_mixer(cardnumber);
	}

	if (access(resfile, F_OK) == 0) {
		save_mixer(cardnumber);
	} else {
		restore_mixer(cardnumber);
	}
}

static void find_devices(struct udev *udev)
{
	struct udev_enumerate *enumerate;
	struct udev_list_entry *list_entry;

	enumerate = udev_enumerate_new(udev);
	udev_enumerate_add_match_property(enumerate, "ALSA_CONTROL", "*");
	udev_enumerate_scan_devices(enumerate);
	udev_list_entry_foreach(list_entry, udev_enumerate_get_list_entry(enumerate)) {
		struct udev_device *device;
		const char *cardnumber;

		device = udev_device_new_from_syspath(udev_enumerate_get_udev(enumerate),
						      udev_list_entry_get_name(list_entry));
		if (device == NULL)
			continue;
		cardnumber = udev_device_get_property_value(device, "ALSA_NUMBER");
		if (cardnumber == NULL)
			continue;
		control(cardnumber);
		udev_device_unref(device);
	}
	udev_enumerate_unref(enumerate);
}

int main(int argc, char *argv[])
{
	struct udev *udev;
	char *cardnumber;

	udev = udev_new();
	strlcpy(resdir, "/run/udev", sizeof(resdir));
	strlcat(resdir, "/alsa", sizeof(resdir));

	mkdir(resdir, 0755);

	cardnumber = getenv("ALSA_NUMBER");
	if (cardnumber != NULL) {
		control(cardnumber);
	} else {
		find_devices(udev);
	}

	udev_unref(udev);
	return 0;
}
