/*
 * Copyright (C) 2009 Valery Inozemtsev <shrek@altlinux.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details:
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <asoundlib.h>
#include "alsactl.h"

typedef struct _CHANNEL {

  char* name;
  int volume;
  int mute;
} CHANNEL;

CHANNEL channels[] = {
/*	  name	vol	mute */
	{"Master", 75, 1},
	{"PCM", 90, 1},
	{"Front", 100, 1},
	{"Synth", 90, 1},
	{"CD", 90, 1},
/* mute mic */
	{"Mic", 0, 0},
/* ESS 1969 chipset has 2 PCM channels */
	{"PCM,1", 90, 1},
/* Trident/YMFPCI/emu10k1 */
	{"Wave", 100, 1},
	{"Music", 100, 1},
	{"AC97", 100, 1},
/* CS4237B chipset */
	{"Master Digital", 75, 1},
/* Envy24 chips with analog outs */
	{"DAC", 90, 1},
	{"DAC,0", 90, 1},
	{"DAC,1", 90, 1},
/* some notebooks use headphone instead of master */
	{"Headphone", 75, 1},
	{"Playback", 100, 1},
	{"Speaker", 75, 1},
	{"Beep", 90, 1}
};

void set_mixer_default(char *cardname)
{
	int i, c;
	char card[32];
	snd_mixer_t *mixer_handle;
	snd_mixer_selem_id_t *sid;
	snd_mixer_elem_t *elem;
	long playback_vol_min, playback_vol_max, set_vol;

	sprintf(card, "hw:%s", cardname);

	if (snd_mixer_open(&mixer_handle, 0) < 0)
		return;
	if (snd_mixer_attach(mixer_handle, card) < 0)
		goto out;
	if (snd_mixer_selem_register(mixer_handle, NULL, NULL) < 0)
		goto out;
	if (snd_mixer_load(mixer_handle) < 0)
		goto out;

	snd_mixer_selem_id_alloca(&sid);
	snd_mixer_selem_id_set_index(sid, 0);

	for (i = 0; i < sizeof(channels) / sizeof(channels[0]); i++) {

		snd_mixer_selem_id_set_name(sid, channels[i].name);
		if (!(elem = snd_mixer_find_selem(mixer_handle, sid)))
			continue;

		for (c = 0; c < SND_MIXER_SCHN_LAST; c++) {
			if (!snd_mixer_selem_has_playback_channel(elem, 0)) {
				snd_mixer_selem_id_set_index(sid, c);
				if ((elem = snd_mixer_find_selem(mixer_handle, sid)) == NULL)
					continue;
			}

			snd_mixer_selem_get_playback_volume_range(elem, &playback_vol_min, &playback_vol_max);
			set_vol = ((float)channels[i].volume / 100) * playback_vol_max;

			if (set_vol > playback_vol_max)
				set_vol = playback_vol_max;
			if (set_vol < playback_vol_min)
				set_vol = playback_vol_min;

			snd_mixer_selem_set_playback_volume(elem, c, set_vol);

			if (snd_mixer_selem_has_playback_switch(elem))
				snd_mixer_selem_set_playback_switch(elem, c, channels[i].mute);
		}
	}

out:
	snd_mixer_close(mixer_handle);
}
